/*******************************************************************************
***  This file is part of Qt Mocks.                                          ***
***                                                                          ***
***  Copyright (C) 2018, 2019, 2020, 2021, 2023, 2024                        ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef QT_MOCKS_MOCK_HPP
#error Should not be included directly.
#endif

template<typename ReturnType, typename ... ParameterTypes>
ReturnType qt_mocks::internals::invokeImplementation(Mock<ReturnType, ParameterTypes ...>& mock)
{
	if constexpr (std::is_void_v<ReturnType>)
	{
		if (mock.function_ == nullptr)
		{
			return;
		}
		std::apply(mock.function_, mock.arguments_);
		return;
	}
	else if constexpr (std::is_reference_v<ReturnType>)
	{
		if (mock.function_ == nullptr)
		{
			static std::remove_reference_t<ReturnType> dummy{};
			return dummy;
		}
		const ReturnType toReturn{std::apply(mock.function_, mock.arguments_)};
		return toReturn;
	}
	else
	{
		if (mock.function_ == nullptr)
			return ReturnType{};
		ReturnType toReturn{std::apply(mock.function_, mock.arguments_)};
		return toReturn;
	}
}
