/*******************************************************************************
***  This file is part of Qt Mocks.                                          ***
***                                                                          ***
***  Copyright (C) 2018, 2019, 2020, 2021, 2023, 2024                        ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef QT_MOCKS_OUTPUT_HPP_
#error Should not be included directly.
#endif


template<typename KeyType, typename ElementType>
std::ostream& qt_mocks::operator<<(
	std::ostream& stream,
	const std::map<KeyType, ElementType>& container
)
{
	internal::outputStream(stream, container.cbegin(), container.cend());
	return stream;
}

template<typename FirstValueType, typename SecondValueType>
std::ostream& qt_mocks::operator<<(
	std::ostream& stream,
	const std::pair<FirstValueType, SecondValueType>& p
)
{
	return stream << p.first << ": " << p.second;
}

template<typename ElementType>
std::ostream& qt_mocks::operator<<(
	std::ostream& stream,
	const std::set<ElementType>& container
)
{
	internal::outputStream(stream, container.cbegin(), container.cend());
	return stream;
}

template<typename ElementType>
std::ostream& qt_mocks::operator<<(
	std::ostream& stream,
	const std::vector<ElementType>& container
)
{
	internal::outputStream(stream, container.cbegin(), container.cend());
	return stream;
}

template<typename KeyType, typename ElementType>
std::ostream& qt_mocks::operator<<(
	std::ostream& stream,
	const ::QMap<KeyType, ElementType>& container
)
{
	internal::outputStream(stream, container.cbegin(), container.cend());
	return stream;
}

template<typename ElementType>
std::ostream& qt_mocks::operator<<(std::ostream& stream, const ::QSet<ElementType>& container)
{
	internal::outputStream(stream, container.cbegin(), container.cend());
	return stream;
}

template<typename ElementType>
std::ostream& qt_mocks::operator<<(std::ostream& stream, const ::QList<ElementType>& container)
{
	internal::outputStream(stream, container.cbegin(), container.cend());
	return stream;
}

template<typename IteratorType>
void qt_mocks::internal::outputStream(
	std::ostream& stream,
	const IteratorType& begin,
	const IteratorType& end
)
{
	stream << '(';
	bool isFirst{true};

	for (auto element{begin} ; element != end ; element = std::next(element, 1))
		stream << (isFirst ? isFirst = false, "" : ", ") << *element;
	stream << ')';
}
